package com.fastest.pass.home.presentation.ui.components

import android.util.Log
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.BoxScope
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.ColumnScope
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.defaultMinSize
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.imePadding
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusManager
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.platform.LocalSoftwareKeyboardController
import androidx.compose.ui.platform.SoftwareKeyboardController
import androidx.compose.ui.res.colorResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.OffsetMapping
import androidx.compose.ui.text.input.TransformedText
import androidx.compose.ui.text.input.VisualTransformation
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.fastest.pass.R
import com.fastest.pass.helpers.BasePreferenceHelper
import com.fastest.pass.home.data.model.NewItemFormField

@Composable
fun AddPaymentCardFormScreen(
    basePreferenceHelper: BasePreferenceHelper,
    onPaymentCardFormList: (ArrayList<NewItemFormField>, Boolean) -> Unit
) {
    val keyboardController = LocalSoftwareKeyboardController.current
    val focusManager = LocalFocusManager.current

    var payment_cards_title by remember { mutableStateOf("") }
    var payment_cards_name_on_card by remember { mutableStateOf("") }
    var payment_cards_type by remember { mutableStateOf("") }
    var payment_cards_number by remember { mutableStateOf("") }
    var payment_cards_security_code by remember { mutableStateOf("") }
    var payment_cards_start_date by remember { mutableStateOf("") }
    var payment_cards_expiration_date by remember { mutableStateOf("") }
    var payment_cards_notes by remember { mutableStateOf("") }

    Box(
        modifier = Modifier
            .fillMaxSize()
            .padding(horizontal = 30.dp)
            .padding(bottom = 20.dp)
            .background(Color.Transparent)
            .imePadding(),
    ) {
        SaveButtonAPCFS(
            buttonText = R.string.save,
            onSaveButtonClick = {
                val paymentCardFormList = ArrayList<NewItemFormField>()
                val saltKey = basePreferenceHelper.getCustomSaltKey()
                val secretKey = basePreferenceHelper.getCustomSecretKey()
                val isTitleEmpty = payment_cards_title.isEmpty()

                payment_cards_title = encryptIfNotEmpty(payment_cards_title, secretKey, saltKey)
                payment_cards_name_on_card = encryptIfNotEmpty(payment_cards_name_on_card, secretKey, saltKey)
                payment_cards_type = encryptIfNotEmpty(payment_cards_type, secretKey, saltKey)
                payment_cards_number = encryptIfNotEmpty(payment_cards_number, secretKey, saltKey)
                payment_cards_security_code = encryptIfNotEmpty(payment_cards_security_code, secretKey, saltKey)
                payment_cards_start_date = encryptIfNotEmpty(payment_cards_start_date, secretKey, saltKey)
                payment_cards_expiration_date = encryptIfNotEmpty(payment_cards_expiration_date, secretKey, saltKey)
                payment_cards_notes = encryptIfNotEmpty(payment_cards_notes, secretKey, saltKey)

                paymentCardFormList.add(NewItemFormField("payment-cards_title", payment_cards_title))
                paymentCardFormList.add(NewItemFormField("payment-cards_name_on_card", payment_cards_name_on_card))
                paymentCardFormList.add(NewItemFormField("payment-cards_type", payment_cards_type))
                paymentCardFormList.add(NewItemFormField("payment-cards_number", payment_cards_number))
                paymentCardFormList.add(NewItemFormField("payment-cards_security_code", payment_cards_security_code))
                paymentCardFormList.add(NewItemFormField("payment-cards_start_date", payment_cards_start_date))
                paymentCardFormList.add(NewItemFormField("payment-cards_expiration_date", payment_cards_expiration_date))
                paymentCardFormList.add(NewItemFormField("payment-cards_notes", payment_cards_notes))

                onPaymentCardFormList.invoke(paymentCardFormList, isTitleEmpty)
            }
            )

        Column(
            modifier = Modifier
                .fillMaxSize()
                .padding(horizontal = 0.dp)
                .padding(bottom = 80.dp)
                .background(Color.Transparent)
                .verticalScroll(rememberScrollState())
        ) {
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAPCFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.title,
                onText = { payment_cards_title = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAPCFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.name_on_card,
                onText = { payment_cards_name_on_card = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NameTextFieldAPCFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.type,
                onText = { payment_cards_type = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NumberTextFieldAPCFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.number,
                placeholder = "1234 1234 1234 1234",
                onText = { payment_cards_number = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NumberTextFieldAPCFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.security_code,
                placeholder = "CVV",
                onText = { payment_cards_security_code = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NumberDateFieldAPCFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.start_date,
                placeholder = "MM/YY",
                onText = { payment_cards_start_date = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NumberDateFieldAPCFS(
                keyboardController = keyboardController,
                focusManager = focusManager,
                labelText = R.string.expiration_date,
                placeholder = "MM/YY",
                onText = { payment_cards_expiration_date = it }
            )
            Spacer(modifier = Modifier.height(20.dp))
            NotesTextFieldAPCFS(
                keyboardController,
                focusManager,
                onText = { payment_cards_notes = it }
            )
        }
    }
}


@Composable
fun ColumnScope.NameTextFieldAPCFS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    labelText: Int,
    onText: (String) -> Unit
) {
    var text by remember { mutableStateOf("") }

    TextField(
        value = text,
        onValueChange = {
            text = it
            onText.invoke(text)
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent)),
        shape = RoundedCornerShape(16.dp),
//        placeholder = {
//            Text(
//                text = stringResource(id = R.string.enter_email_address),
//                color = colorResource(id = R.color.gray_splash),
//                style = MaterialTheme.typography.displayMedium
//            )
//        },
        label = {
            Text(text = stringResource(id = labelText),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
//        leadingIcon = {
//            Image(
//                painter = painterResource(id = R.drawable.profile_circle),
//                contentDescription = "Title Logo",
//                modifier = Modifier
//                    .size(24.dp, 24.dp)
//            )
//        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Text,
            imeAction = ImeAction.Next
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
    )
}

@Composable
fun ColumnScope.NumberTextFieldAPCFS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    labelText: Int,
    placeholder: String,
    onText: (String) -> Unit
) {
    var text by remember { mutableStateOf("") }

    TextField(
        value = text,
        onValueChange = {
            text = it
            onText.invoke(text)
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent)),
        shape = RoundedCornerShape(16.dp),
//        placeholder = {
//            Text(
//                text = placeholder,
//                style = MaterialTheme.typography.titleSmall.copy(
//                    color = colorResource(id = R.color.gray_text)
//                )
//            )
//        },
        label = {
            Text(text = stringResource(id = labelText),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
//        leadingIcon = {
//            Image(
//                painter = painterResource(id = R.drawable.profile_circle),
//                contentDescription = "Title Logo",
//                modifier = Modifier
//                    .size(24.dp, 24.dp)
//            )
//        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Number,
            imeAction = ImeAction.Next
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
    )
}

@Composable
fun ColumnScope.NumberDateFieldAPCFS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    labelText: Int,
    placeholder: String,
    onText: (String) -> Unit
) {
    var text by remember { mutableStateOf("") }
    val maxLength = 4

    TextField(
        value = text,
        onValueChange = { input ->
            if (input.length <= maxLength) {
                text = input.filter { it.isDigit() }
                onText.invoke(text)
            }
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent)),
        shape = RoundedCornerShape(16.dp),
        placeholder = {
            Text(
                text = placeholder,
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
        label = {
            Text(text = stringResource(id = labelText),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Number,
            imeAction = ImeAction.Next
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
        visualTransformation = DateTransformation()
    )
}

fun validateMMYY(input: String): Boolean {
    val parts = input.split("/")
    if (parts.size != 2) return false
    val month = parts[0].toIntOrNull() ?: return false
    val year = parts[1].toIntOrNull() ?: return false
    return month in 1..12 && parts[1].length == 2
}

class DateTransformation() : VisualTransformation {
    override fun filter(text: AnnotatedString): TransformedText {
        return dateFilter(text)
    }
}

fun dateFilter(text: AnnotatedString): TransformedText {

    val trimmed = if (text.text.length >= 4) text.text.substring(0..3) else text.text
    var out = ""
    for (i in trimmed.indices) {
        out += trimmed[i]
        if (i == 1 && trimmed.length > 2) out += "/"
    }

    val numberOffsetTranslator = object : OffsetMapping {
        override fun originalToTransformed(offset: Int): Int {
            if (offset <= 2) return offset
            if (offset <= 4) return offset +1
            return 5
        }

        override fun transformedToOriginal(offset: Int): Int {
            if (offset <=2) return offset
            if (offset <=5) return offset -1
            return 4
        }
    }

    return TransformedText(AnnotatedString(out), numberOffsetTranslator)
}

@Composable
fun ColumnScope.NotesTextFieldAPCFS(
    keyboardController: SoftwareKeyboardController?,
    focusManager: FocusManager,
    onText: (String) -> Unit
) {
    var notesText by remember { mutableStateOf("") }

    TextField(
        value = notesText,
        onValueChange = {
            notesText = it
            onText.invoke(notesText)
        },
        textStyle = MaterialTheme.typography.displayMedium,
        modifier = Modifier
            .align(Alignment.Start)
            .fillMaxWidth()
            .defaultMinSize(minHeight = 60.dp)
            .wrapContentHeight()
            .border(
                1.dp,
                color = colorResource(id = R.color.gray_border_textfield),
                shape = RoundedCornerShape(16.dp)
            )
            .background(color = colorResource(id = R.color.transparent))
        ,
        shape = RoundedCornerShape(16.dp),
//        placeholder = {
//            Text(
//                text = stringResource(id = R.string.enter_email_address),
//                color = colorResource(id = R.color.gray_splash),
//                style = MaterialTheme.typography.displayMedium
//            )
//        },
        label = {
            Text(text = stringResource(R.string.notes),
                style = MaterialTheme.typography.titleSmall.copy(
                    color = colorResource(id = R.color.gray_text)
                )
            )
        },
//        leadingIcon = {
//            Image(
//                painter = painterResource(id = R.drawable.profile_circle),
//                contentDescription = "Title Logo",
//                modifier = Modifier
//                    .size(24.dp, 24.dp)
//            )
//        },
        colors = TextFieldDefaults.colors(
            focusedLabelColor = colorResource(id = R.color.gray_splash),
            unfocusedContainerColor = colorResource(id = R.color.transparent),
            focusedContainerColor = colorResource(id = R.color.transparent),
            focusedIndicatorColor = colorResource(id = R.color.transparent),
            disabledIndicatorColor = colorResource(id = R.color.transparent),
            unfocusedIndicatorColor = colorResource(id = R.color.transparent),
            cursorColor = colorResource(id = R.color.gray_splash),
        ),
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Text,
            imeAction = ImeAction.Default
        ),
        keyboardActions = KeyboardActions(
            onDone = {
                focusManager.clearFocus()
                keyboardController?.hide()
            }
        ),
    )
}

@Composable
fun BoxScope.SaveButtonAPCFS(
    buttonText: Int,
    onSaveButtonClick: () -> Unit
    ) {
    Button(
        modifier = Modifier
            .background(colorResource(id = R.color.transparent))
            .fillMaxWidth()
            .height(60.dp)
            .align(Alignment.BottomCenter)
            .clickable() { },
        onClick = {
            onSaveButtonClick.invoke()
        },
        shape = RoundedCornerShape(15.dp),
//            border = BorderStroke(25.dp, colorResource(id = R.color.black)),
        colors = ButtonDefaults.buttonColors(
            contentColor = colorResource(id = R.color.white),
            containerColor = colorResource(id = R.color.red_login_button),
        ),
    )
    {
        Text(
            text = stringResource(id = buttonText),
            style = MaterialTheme.typography.bodyMedium.copy(
                fontSize = 20.sp,
                color = colorResource(id = R.color.white)
            ),
            textAlign = TextAlign.Center
        )
    }
}